package cleii.data;


/*
 *                            HashTable *
 *                                / \
 *                               /   \
 *                              /     \
 *                    ListHashTable   IntenalHashTable *
 *                                            / \
 *                                           /   \
 *                                          /     \
 *                   LinearInspectionHashTable    DoubleHashingHashTable
 *                   
 *  Gerarchia delle classi per le tabelle Hash. L'* indica classe astratta                                  
 *  
*/ 

public abstract class HashTable <T> {
	protected int k;    //dimensione della tabella Hash
	protected int size; //numero di elementi presenti in tabella
	private double maxLoad; //massimo fattore di carico
	
	public HashTable (int k, double maxLoad) {
		//voglio creare le tabelle di dimensione un numero primo
		this.k = findPrime(k);
		this.maxLoad = maxLoad;
		size=0;
	}
	
	public int getK() {
		return k;
	}
	
	public int size() {
		return size;
	}
	
	public double getMaxLoad() {
		return maxLoad;
	}
	
	
	/*
	 * Operazioni della tabella Hash (metodi astratti)
	 */
	
	public abstract void insert (T x);
	
	public abstract T search (T x);
	
	public abstract void delete (T x);
	
	
	
	static protected boolean isPrime (int n) {
		for (int i=2; i*i<=n; i++) {
			if (n%i==0) {
				return false;
			}
		}
		return true;
	}
	
	/*
	 * metodo che restituisce il numero primo pi piccolo che sia maggiore o uguale a n
	 */
	static protected int findPrime (int n) {
		while (true) {
			if (isPrime(n)) return n;
			n++;
		}
	}
	

}
